@@ -19,7 +19,6 @@
 from magneticalc.Norm_Types import *
 from typing import Union, List
 import numpy as np
-from magneticalc.Debug import Debug
 from magneticalc.Metric import metric_norm
 from magneticalc.Comparison_Types import *
 
@@ -27,8 +26,8 @@ from magneticalc.Comparison_Types import *
 class Constraint:
     """ Constraint class. """
 
-    # Supported norm types
-    Norm_Types_List = [
+    """ Supported norm types. """
+    Norm_Types_Supported = [
         NORM_TYPE_X,
         NORM_TYPE_Y,
         NORM_TYPE_Z,
@@ -43,20 +42,16 @@ class Constraint:
         NORM_TYPE_ANGLE_XZ,
         NORM_TYPE_ANGLE_YZ
     ]
-    Norm_Types_List_Str = [norm_type_to_str(norm_type) for norm_type in Norm_Types_List]
 
-    # Supported norm types using minimum and maximum angles in degrees
-    Norm_Types_Degrees_List = [
+    """ Supported norm types using minimum and maximum angles in degrees. """
+    Norm_Types_Supported_Degrees = [
         NORM_TYPE_ANGLE_XY,
         NORM_TYPE_ANGLE_XZ,
         NORM_TYPE_ANGLE_YZ
     ]
 
-    # Supported comparison types
-    Comparison_Types_List = [
-        COMPARISON_TYPE_IN_RANGE
-    ]
-    Comparison_Types_List_Str = [comparison_type_to_str(comparison_type) for comparison_type in Comparison_Types_List]
+    """ Supported comparison types. """
+    Comparison_Types_Supported = list(Comparison_Types_Names_Map.keys())
 
     def __init__(self, norm_type: int, comparison_type: int, _min: float, _max: float, permeability: float):
         """
@@ -68,15 +63,8 @@ class Constraint:
         @param _max: Maximum value
         @param permeability: Relative permeability µ_r
         """
-        if norm_type not in self.Norm_Types_List:
-            Debug(self, ": ERROR: Invalid norm ID", error=True)
-            return
-
-        if comparison_type not in self.Comparison_Types_List:
-            Debug(self, ": ERROR: Invalid comparison ID", error=True)
-            return
-
-        self._is_angle = norm_type in self.Norm_Types_Degrees_List
+        norm_type = norm_type_safe(norm_type)
+        comparison_type = comparison_type_safe(comparison_type)
 
         self._norm_type = norm_type
         self._comparison_type = comparison_type
@@ -84,6 +72,19 @@ class Constraint:
         self._min = _min
         self._max = _max
 
+        if norm_type in self.Norm_Types_Supported_Degrees:
+            # Clip angles to range
+            self._min %= 360
+            self._max %= 360
+
+            # Normalize angles
+            self._min /= 360.0
+            self._max /= 360.0
+
+            # Ensure monotonicity
+            if self._min > self._max:
+                self._min, self._max = self._max, self._min
+
         self.permeability = permeability
 
     def evaluate(self, point: Union[np.ndarray, List[float]]) -> bool:
@@ -94,15 +95,14 @@ class Constraint:
         """
         norm = metric_norm(self._norm_type, point)
 
-        # Perform comparison
         if self._comparison_type == COMPARISON_TYPE_IN_RANGE:
 
-            if self._is_angle:
-                # Convert normalized angle to degrees
-                norm *= 360
-
             return self._min <= norm <= self._max
 
+        elif self._comparison_type == COMPARISON_TYPE_NOT_IN_RANGE:
+
+            return not self._min <= norm <= self._max
+
         else:
             # Invalid comparison ID
             return False
