@@ -21,11 +21,11 @@ from typing import Optional
 from functools import partial
 from PyQt5.QtWidgets import QHBoxLayout, QVBoxLayout, QSizePolicy
 from PyQt5.QtWidgets import QButtonGroup, QCheckBox, QRadioButton
-from magneticalc.QDoubleSpinBox2 import QDoubleSpinBox2
-from magneticalc.QGroupBox2 import QGroupBox2
-from magneticalc.QHLine import QHLine
-from magneticalc.QIconLabel import QIconLabel
-from magneticalc.QLabel2 import QLabel2
+from magneticalc.QtWidgets2.QDoubleSpinBox2 import QDoubleSpinBox2
+from magneticalc.QtWidgets2.QGroupBox2 import QGroupBox2
+from magneticalc.QtWidgets2.QHLine import QHLine
+from magneticalc.QtWidgets2.QIconLabel import QIconLabel
+from magneticalc.QtWidgets2.QLabel2 import QLabel2
 from magneticalc.Debug import Debug
 from magneticalc.Field_Types import Field_Types_Names_Map
 from magneticalc.Metric import Metric
@@ -51,13 +51,13 @@ class Field_Widget(QGroupBox2):
 
         @param gui: GUI
         """
-        QGroupBox2.__init__(self, "Field")
+        QGroupBox2.__init__(self, "Field", color=Theme.DarkColor)
         Debug(self, ": Init", init=True)
         self.gui = gui
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.addLayout(QIconLabel("Type", "mdi.tune-variant"))
+        self.addLayout(QIconLabel("Type", "mdi.tune-variant", color=Theme.DarkColor))
 
         field_type_layout_left = QVBoxLayout()
         field_type_layout_right = QVBoxLayout()
@@ -92,7 +92,7 @@ class Field_Widget(QGroupBox2):
 
         self.addWidget(QHLine())
 
-        self.addLayout(QIconLabel("Distance Limit", "mdi.ruler"))
+        self.addLayout(QIconLabel("Distance Limit", "mdi.ruler", color=Theme.DarkColor))
         self.distance_limit_spinbox = QDoubleSpinBox2(
             gui=self.gui,
             minimum=self.DistanceLimitMin,
@@ -150,8 +150,8 @@ class Field_Widget(QGroupBox2):
         Updates the labels.
         """
         if self.gui.model.field.valid:
-            self.total_calculations_label.setText(str(self.gui.model.field.get_total_calculations()))
-            self.total_skipped_calculations_label.setText(str(self.gui.model.field.get_total_skipped_calculations()))
+            self.total_calculations_label.setText(str(self.gui.model.field.total_calculations))
+            self.total_skipped_calculations_label.setText(str(self.gui.model.field.total_skipped_calculations))
         else:
             self.total_calculations_label.setText("N/A")
             self.total_skipped_calculations_label.setText("N/A")
@@ -163,7 +163,7 @@ class Field_Widget(QGroupBox2):
         for field_type in Field_Types_Names_Map:
             self.field_type_checkboxes[field_type].setChecked(self.gui.model.get_valid_field(field_type) is not None)
 
-        self.indicate_valid(self.gui.model.field.valid)
+        self.set_color(Theme.MainColor if self.gui.model.field.valid else Theme.FailureColor)
 
     # ------------------------------------------------------------------------------------------------------------------
 
@@ -208,12 +208,10 @@ class Field_Widget(QGroupBox2):
         with ModelAccess(self.gui, recalculate):
 
             field_type = self.gui.project.set_get_int("field_type", _field_type_)
-            backend_type = self.gui.project.get_int("backend_type")
             distance_limit = self.gui.project.set_get_float("field_distance_limit", _distance_limit_)
 
             self.gui.model.set_field(
                 invalidate=invalidate,
-                backend_type=backend_type,
                 field_type=field_type,
                 distance_limit=distance_limit,
                 length_scale=Metric.LengthScale
